package org.ids_mannheim;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicIntegerArray;
import java.util.stream.IntStream;

public class FoldedEntry implements Comparable<FoldedEntry> {
    static int FOLDS = 10;
    final AtomicIntegerArray count;

    public static void setFolds(int folds) {
        FOLDS = folds;
    }

    public FoldedEntry() {
        count = new AtomicIntegerArray(FOLDS + 1);
    }

    @Override
    public int compareTo(FoldedEntry foldedEntry) {
        if (foldedEntry == null) {
            return -1;
        } else {
            return Integer.compareUnsigned(count.get(0), foldedEntry.count.get(0));
        }
    }

    public static void incr(ConcurrentHashMap<String, AtomicInteger> map, String ngram) {
        map.compute(ngram, (key, value) -> {
            if (value == null) {
                value = new AtomicInteger();
            }
            value.incrementAndGet();
            return value;
        });
    }

    public static void incr(ConcurrentHashMap<String, FoldedEntry> map, String ngram, int fold) {
        map.compute(ngram, (key, value) -> {
            if (value == null) {
                value = new FoldedEntry();
            }
            value.count.incrementAndGet(fold);
            value.count.incrementAndGet(0);
            return value;
        });
    }

    @Override
    public String toString() {
        StringBuilder b = new StringBuilder();
        IntStream.rangeClosed(1, FOLDS).forEach(i -> b.append("\t").append(Integer.toUnsignedString(count.get(i))));
        b.append("\t").append(count.get(0));
        return b.toString();
    }
}
